@@ -24,6 +24,9 @@ backends = {'tk': 'TkAgg',
             'nbagg': 'nbAgg',
             'notebook': 'nbAgg',
             'agg': 'agg',
+            'svg': 'svg',
+            'pdf': 'pdf',
+            'ps': 'ps',
             'inline': 'module://ipykernel.pylab.backend_inline',
             'ipympl': 'module://ipympl.backend_nbagg',
             'widget': 'module://ipympl.backend_nbagg',
